
#ifndef VISANPLOT_EXPORT_H
#define VISANPLOT_EXPORT_H

#ifdef VISANPLOT_STATIC_DEFINE
#  define VISANPLOT_EXPORT
#  define VISANPLOT_NO_EXPORT
#else
#  ifndef VISANPLOT_EXPORT
#    ifdef plot_EXPORTS
        /* We are building this library */
#      define VISANPLOT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VISANPLOT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VISANPLOT_NO_EXPORT
#    define VISANPLOT_NO_EXPORT 
#  endif
#endif

#ifndef VISANPLOT_DEPRECATED
#  define VISANPLOT_DEPRECATED __declspec(deprecated)
#endif

#ifndef VISANPLOT_DEPRECATED_EXPORT
#  define VISANPLOT_DEPRECATED_EXPORT VISANPLOT_EXPORT VISANPLOT_DEPRECATED
#endif

#ifndef VISANPLOT_DEPRECATED_NO_EXPORT
#  define VISANPLOT_DEPRECATED_NO_EXPORT VISANPLOT_NO_EXPORT VISANPLOT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VISANPLOT_NO_DEPRECATED
#    define VISANPLOT_NO_DEPRECATED
#  endif
#endif

#endif /* VISANPLOT_EXPORT_H */
